import benchmarkParameters as bp
import createGNNetworks as gn
import utilities as ut
import importlib
import numpy as np
from multiprocessing import Process, Manager, Pool
import multiprocessing
import json
from tqdm.auto import tqdm
import pandas as pd
importlib.reload(ut)
import matplotlib.gridspec as gridspec
from adjustText import adjust_text;
import matplotlib
#Disable Matplotlib interface
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from matplotlib import patheffects
import igraph as ig


matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42
plt.rc('font', family='sans-serif')
# plt.rc('text', usetex=True)
plt.rc('xtick', labelsize=15)
plt.rc('ytick', labelsize=15)
plt.rc('axes', labelsize=16)
plt.rc('legend', fontsize=14)

categoriesInOrder = [
 'Biological',
 'Transport-Geographic',
 'Information',
 'Social',
 'Benchmarks_t1-2.00_t2-1.00',
 'GN',
 'Models'
]

variant = "SBMBasic"
rewireMode = "basic"
alphaPoints = 51
alphaValues = np.linspace(0,1.0,alphaPoints);
networkList = ut.getNetworkList()
trivialThreshold = 1.0

with open("../Data/networkProperties.json","rt") as fd:
    networkProperties = json.load(fd)

with open("../Data/majorSizes_%s.json"%rewireMode,"rt") as fd:
    majorSizesByNetwork = json.load(fd)

for entry in tqdm(networkList,leave=False):
    networkName,label,category  =  entry
    majorSizes = []
    communitySizes = ut.loadDistributionData("CS","%s_%s"%(rewireMode,variant),networkName)
    numberOfCommunities = ut.loadDistributionData("NC","%s_%s"%(rewireMode,variant),networkName)
    if(communitySizes is None or numberOfCommunities is None or networkName not in networkProperties):
        continue;
    verticesCount = networkProperties[networkName]["Node count"]
    majorCommunitySizes = [];
    errorMajorCommunitySizes = [];
    trivialRatios = [];
    for alphaIndex in range(len(numberOfCommunities)):
        communityIndices = np.cumsum(numberOfCommunities[alphaIndex]);
        majorCommunitySizesDistrib=[];
        errorMajorCommunitySizesDistrib=[];
        trivialCount = 0;
        for i in range(len(communityIndices)):
            if(i>0):
                startIndex = int(communityIndices[i-1]);
            else:
                startIndex=0;
            endIndex = int(communityIndices[i]);
            if(networkName in majorSizesByNetwork):
                trivialMax = majorSizesByNetwork[networkName][alphaIndex][i]
            else:
                trivialMax = verticesCount
            # assert (trivialMax<=verticesCount)
                
            communitySizesDistrib = communitySizes[alphaIndex][startIndex:endIndex];
            majorCommunitySizesDistrib.append(np.max(communitySizesDistrib));
            trivialThresholdSize = trivialThreshold*trivialMax;
            if(np.max(communitySizesDistrib)>=trivialThresholdSize):
                trivialCount+=1;
            elif(len(communitySizesDistrib)>=trivialThresholdSize):
                trivialCount+=1;
        trivialRatios.append(trivialCount/len(communityIndices));
    networkProperties[networkName]["TPR Data"] = trivialRatios;
    networkProperties[networkName]["TPR Area"] = (1.0-np.trapz(trivialRatios, alphaValues))




df = pd.DataFrame.from_dict(networkProperties).T
df["Category"] = df["Category"].replace("Economy","Information") #merged economy and Information
# fig, ax = plt.subplots(1, 1, figsize=(9, 9))

df["Modularity Diff"] = df["Modularity"] - df["ERModularity"]
df["ModularityCPM Diff"] = df["ModularityCPM"] - df["ERModularityCPM"]
df["ModularityRBER Diff"] = df["ModularityRBER"] - df["ERModularityRBER"]
df["ModularitySignificance Diff"] = df["ModularitySignificance"] - df["ERModularitySignificance"]
df["ModularitySurprise Diff"] = df["ModularitySurprise"] - df["ERModularitySurprise"]


df["Modularity Config Diff"] = df["Modularity"] - df["ConfigModularity"]
df["ModularityCPM Config Diff"] = df["ModularityCPM"] - df["ConfigModularityCPM"]
df["ModularityRBER Config Diff"] = df["ModularityRBER"] - df["ConfigModularityRBER"]
df["ModularitySignificance Config Diff"] = df["ModularitySignificance"] - df["ConfigModularitySignificance"]
df["ModularitySurprise Config Diff"] = df["ModularitySurprise"] - df["ConfigModularitySurprise"]


df["Modularity Z-Score"] = (df["Modularity"] - df["ConfigModularityAverage"])/df["ConfigModularityStd"]

df["MDL-DC Diff. By Sum."] = (df["MDL-DC SBM"] - df["MDL-DC Single"]) / (df["MDL-DC SBM"] + df["MDL-DC Single"])
df["MDL-DC One Minus Ratio"] = 1.0 - df["MDL-DC Ratio"]
df["MDL-DC Ratio"] = 1.0/df["MDL-DC Ratio"]
# df["Modularity Norm"] = df["Modularity Diff"]/df["Modularity"]
# df["ModularityCPM Norm"] = df["ModularityCPM Diff"]/df["ModularityCPM"]
# df["ModularityRBER Norm"] = df["ModularityRBER Diff"]/df["ModularityRBER"]
# df["ModularitySignificance Norm"] = df["ModularitySignificance Diff"]/df["ModularitySignificance"]
# df["ModularitySurprise Norm"] = df["ModularitySurprise Diff"]/df["ModularitySurprise"]


fig = plt.figure(figsize=(9, 9))
ax = fig.gca()
cmap = matplotlib.cm.tab10
grouped = df.groupby('Category')
for index,(key, group) in enumerate(grouped):
    group.plot.scatter(x="Modularity",y="TPR Area",
    color=cmap(index),
    ax=ax,
    label=key,
    s=30)\
        .legend(loc='center left',bbox_to_anchor=(1.0, 0.5))

fig.savefig("../Figures/AllTPR_%s.pdf"%rewireMode)
plt.close(fig)


fig, ax = plt.subplots(1, 1, figsize=(9, 9))
cmap = matplotlib.cm.tab10
grouped = df.groupby('Category')
for index,(key, group) in enumerate(grouped):
    group.plot.scatter(x="Average degree",y="TPR Area",color=cmap(index),ax=ax,label=key)

ax.set_xscale("log")
fig.savefig("../Figures/AllTPRvsAvgDegree_%s.pdf"%rewireMode)
plt.close(fig)


fig, ax = plt.subplots(1, 1, figsize=(9, 9))
cmap = matplotlib.cm.tab10
grouped = df.groupby('Category')
for index,(key, group) in enumerate(grouped):
    group.plot.scatter(x="Node count",y="TPR Area",color=cmap(index),ax=ax,label=key)

ax.set_xscale("log")
fig.savefig("../Figures/AllTPRvsSize_%s.pdf"%rewireMode)
plt.close(fig)


dforig = df



from scipy import stats
# firstProperty = "MDL-DC Ratio"
# firstProperty = "MDL-DC One Minus Ratio"
# firstProperty = "TPR Area"
firstProperty = "Modularity Config Diff"
# firstProperty = "Modularity"
useRanks = False

removeNetworks = set([
    # "bench_n5000_t2.000_T1.000_mu0.700_minc20_maxc500_k20.000_maxk250-r0",
    # "bench_n5000_t2.000_T1.000_mu0.800_minc20_maxc500_k20.000_maxk250-r0",
    # "bench_n5000_t2.000_T1.000_mu0.900_minc20_maxc500_k20.000_maxk250-r0",
    # "bench_n5000_t2.000_T1.000_mu1.000_minc20_maxc500_k20.000_maxk250-r0"
    ])

df = dforig[~dforig.index.isin(removeNetworks)].dropna().copy()
# df["MDL-DC Diff. Norm."] = -df["MDL-DC Diff. Norm."]
for secondProperty in [
    "MDL-DC Diff. Norm.",
    "MDL-nonDC Diff. Norm.",
    "MDL-DC Diff. By Sum.",
    # "MDL-nonDC Diff. Norm.Avg.",
    "Average degree",
    "Node count",
    "Edges count",
    "Modularity",
    # "ModularityCPM",
    # "ModularityRBER",
    # "ModularitySignificance",
    # "ModularitySurprise",
    "Modularity Diff",
    "Modularity Config Diff",
    "Modularity Z-Score",
    "MDL-DC Ratio",
    "MDL-DC One Minus Ratio",
    "TPR Area",
    # "ModularityCPM Diff.",
    # "ModularityRBER Diff.",
    # "ModularitySignificance Diff.",
    # "ModularitySurprise Diff."
    ]:
    fig, ax = plt.subplots(1, 1, figsize=(9, 9))
    cmap = matplotlib.cm.tab10
    # secondProperty = "TPR Area"
    if(useRanks):
        df[firstProperty+" Ranks"] = df[firstProperty].rank(ascending=False)
        df[secondProperty+" Ranks"] = df[secondProperty].rank(ascending=False)
        firstPropertyFinal = firstProperty+" Ranks"
        secondPropertyFinal = secondProperty+" Ranks"
    else:
        firstPropertyFinal = firstProperty
        secondPropertyFinal = secondProperty
    grouped = df.groupby('Category')
    groupedDictionary = {key: group for key, group in grouped}

    for index,key in enumerate(categoriesInOrder):
        group = groupedDictionary[key]
        # group.plot.scatter(x=firstProperty,y=secondProperty,color=cmap(index),ax=ax,label=key)
        if (key.startswith("Benchmarks") or key == "GN"):
            ax.plot(group[firstPropertyFinal],group[secondPropertyFinal],color=cmap(index),lw=2)
        ax.scatter(group[firstPropertyFinal],group[secondPropertyFinal],label=key,color=cmap(index))

    for label,x,y in zip(df["Label"],df[firstPropertyFinal],df[secondPropertyFinal]):
        if(np.isfinite(x)):
            ax.text(x,y,label,fontsize=6)
    # ax.scatter(group[firstPropertyFinal],group[secondPropertyFinal],label=key,color=cmap(index))
    ax.set_xlabel(firstPropertyFinal)
    ax.set_ylabel(secondPropertyFinal) 
    ax.legend(prop={'size': 6})
    spearmanCoeff = stats.spearmanr(df.dropna()[firstPropertyFinal].dropna(),df.dropna()[secondPropertyFinal]).correlation
    pearsonCoeff = stats.pearsonr(df.dropna()[firstPropertyFinal].dropna(),df.dropna()[secondPropertyFinal])[0]
    ax.set_title("Corr.= %.2f , Spearman: %.2f"%(pearsonCoeff,spearmanCoeff))
    
    print("Spearman: %s x %s: %f"%(firstPropertyFinal,secondPropertyFinal,stats.spearmanr(df.dropna()[firstPropertyFinal].dropna(),df.dropna()[secondPropertyFinal]).correlation))
    print("Pearson: %s x %s: %f"%(firstPropertyFinal,secondPropertyFinal,stats.pearsonr(df.dropna()[firstPropertyFinal].dropna(),df.dropna()[secondPropertyFinal])[0]))
    # ax.set_xscale("log")
    fig.savefig("../Figures/All_%s_%s_vs_%s_.pdf"%(rewireMode,firstPropertyFinal,secondPropertyFinal))
    plt.close(fig)







fig, (ax1,ax2) = plt.subplots(1, 2, figsize=(10,3.5),sharey="all")
benchmarkNames = [];
networkRealization = 0
tau1 = 2.0
tau2 = 1.0
vmin = np.min(bp.muValues)
vmax = np.max(bp.muValues)
cmap=matplotlib.cm.plasma

norm = matplotlib.colors.Normalize(vmin=vmin,vmax=vmax)
for mu in bp.muValues:
    networkName = bp.getNetworkName(mu,networkRealization,tau1,tau2)
    # "$\\mu=%.3f$"%mu
    if("TPR Data" in networkProperties[networkName]):
        trivialRatios = networkProperties[networkName]["TPR Data"]
        ax1.plot(alphaValues,trivialRatios,color = cmap(norm(mu)),label = "$\\mu=%.2f$"%mu)

ax1.legend(fontsize="small",frameon=False)
ax1.set_xlabel("$\\alpha$")
ax1.set_ylabel("TPR")
ax1.text(-0.15, -0.15,'a LFR',
     horizontalalignment='left',
     verticalalignment='center',
     transform = ax1.transAxes,
     path_effects=[patheffects.withStroke(linewidth=3,foreground="w")])

vmin = np.min(gn.muValues)
vmax = np.max(gn.muValues)
norm = matplotlib.colors.Normalize(vmin=vmin,vmax=vmax)
for mu in gn.muValues:
    networkName = gn.getNetworkName(gn.nodesCount,gn.communitiesCount,gn.avgk,mu,networkRealization)
    # "$\\mu=%.3f$"%mu
    if("TPR Data" in networkProperties[networkName]):
        trivialRatios = networkProperties[networkName]["TPR Data"]
        ax2.plot(alphaValues,trivialRatios,color = cmap(norm(mu)),label = "$\\mu=%.2f$"%mu)
ax2.legend(fontsize="small",frameon=False)
ax2.set_xlabel("$\\alpha$")
# ax2.set_ylabel("TPR")


ax2.text(-0.15, -0.15,'b SBM',
     horizontalalignment='left',
     verticalalignment='center',
     transform = ax2.transAxes,
     path_effects=[patheffects.withStroke(linewidth=3,foreground="w")])

plt.tight_layout()
fig.savefig("../Figures/TPR_Benchmarks_%s.pdf"%rewireMode)
plt.close(fig)


# for networkRealization in range(bp.networkRealizations):
#     for mu in gn.muValues:
#         # benchmarkNames.append(gn.getNetworkName(gn.P_in,P_out,networkRealization));
#         benchmarkNames.append((gn.getNetworkName(gn.nodesCount,gn.communitiesCount,gn.avgk,mu,networkRealization),"$\\mu=%.3f$"%mu,"GN"));



# alphaPoints = 51
# alphaValues = np.linspace(0,1.0,alphaPoints);

# networkName = bp.getNetworkName(0.8,networkRealization,tau1,tau2)
# sizes = []
# for alpha in alphaValues[-20:]:
#     for perturbationIndex in range(bp.perturbationRealizations):
#         g = ut.getPerturbedNetwork(networkName,rewireMode,alpha,perturbationIndex)
#         majorSize = g.clusters().giant().vcount()
#         mcs = np.max(list(Counter(g.vs["basic_SBMBasic"]).values()))
#         tpr = mcs/majorSize
#         sizes.append(tpr)
#         if(tpr<1.0):
#             break;
#     if(tpr<1.0):
#         break;
#     print("%g : %.2f ± %.2f"%(alpha,np.average(sizes),np.std(sizes)))
    




# def SBMMinimizeMembership(vertexCount,edges,directed=False):
# 	g = gt.Graph(directed=directed);
# 	for _ in range(0,vertexCount):
# 		g.add_vertex();
		
# 	for edge in edges:
# 		g.add_edge(edge[0],edge[1]);
# 	state = gtInference.minimize.minimize_blockmodel_dl(g,deg_corr=True);
# 	S2 = state.entropy();
# 	B2 = state.get_Be();
# 	return (list(state.get_blocks()),S2,B2);



# networkName = bp.getNetworkName(0.1,networkRealization,tau1,tau2)
# originalNetwork = ut.getPerturbedNetwork(networkName,rewireMode,0.0,perturbationIndex)

# originalNetwork = originalNetwork.simplify().clusters(mode="WEAK").giant()

# for alpha in [0.0]:
#     originalEdges = [(edge.source, edge.target) for edge in originalNetwork.es];
#     vertexCount = originalNetwork.vcount();
#     degrees = originalNetwork.degree();
#     directed = originalNetwork.is_directed()
#     edgesCount = len(originalEdges);
#     newEdges = np.array(originalEdges);
#     selectedEdgesIndices = np.where(np.random.random(edgesCount)<alpha)[0];
#     # plt.figure()
#     # plt.hist(selectedEdgesIndices)
#     # plt.savefig("tests/%s_TEST_Alpha_%.2f_hist.pdf"%(networkName,alpha))
#     # plt.close()
#     generatedSelectedEdges = np.array(np.random.randint(0,vertexCount,(len(selectedEdgesIndices),2)));
#     newEdges[selectedEdgesIndices] = generatedSelectedEdges;
#     newEdges = [(fromIndex,toIndex) for fromIndex,toIndex in newEdges];
#     perturbedNetwork = ig.Graph(n=vertexCount,edges=list(newEdges),directed=False);
#     membership,_,_ = SBMMinimizeMembership(vertexCount,newEdges)
#     perturbedNetwork.vs["SBMBasic"] = [str(entry) for entry in membership]
#     xn.igraph2xnet(perturbedNetwork,"tests/%s_TEST_Alpha_%.2f.xnet"%(networkName,alpha)) 

# networkName = "bio-CE-HT"
# print(networkProperties[networkName]["Node count"])
# print(networkProperties[networkName]["Average degree"])
# print(networkProperties[networkName]["TPR Area"])

# counts =[]
# degrees = []
# tpr = []
# for networkName,data in networkProperties.items():
#     if(networkName.startswith("socfb")):
#         if("TPR Area" in data):
#             counts.append(data["Node count"])
#             degrees.append(data["Average degree"])
#             tpr.append(data["TPR Area"])

# print("socfb")
# print(np.average(counts),"±",np.std(counts))
# print(np.average(degrees),"±",np.std(degrees))
# print(np.average(tpr),"±",np.std(tpr))


import matplotlib.patches as mpl_patches
import pandas as pd


for networkName in networkProperties.keys():
    data = networkProperties[networkName]
    if("TPR Data" not in data):
        print("Skipping %s ..."%networkName)
        continue
    fig = plt.figure(figsize=(3*1.61803398875,3))
    ax = plt.axes((0.2, 0.2, 0.70, 0.70), facecolor='w')
    nodeCount = data["Node count"]
    averageDegree = data["Average degree"]
    TPRArea = data["TPR Area"]
    trivialRatios = data["TPR Data"]
    ax.plot(alphaValues,trivialRatios,color = "#262626",lw=2.0)
    ax.fill_between(alphaValues,trivialRatios,1,color = "#E8EAEA")
    ax.set_xlabel("$\\alpha$")
    ax.set_ylabel("TPR")
    ax.set_title(networkName)
    ax.set_xlim(-0.00,1.02)
    ax.set_ylim(-0.020,1.020)
    
    # create a list with two empty handles (or more if needed)
    handles = [mpl_patches.Rectangle((0, 0), 1, 1, fc="white", ec="white", 
                                    lw=0, alpha=0)] * 3

    # create the corresponding number of labels (= the text you want to display)
    labels = []
    labels.append("$N$ = %d"%nodeCount)
    labels.append("$\\langle k\\rangle$ = %.2f"%averageDegree)
    labels.append("$Q_{r}$ = %.2f"%TPRArea)

    # create the legend, supressing the blank space of the empty line symbol and the
    # padding between symbol and label by setting handlelenght and handletextpad
    ax.legend(handles, labels, loc='best', 
            fancybox=False, framealpha=0, 
            handlelength=0, handletextpad=0)
    ax.spines['right'].set_visible(False)
    ax.spines['top'].set_visible(False)
    for axis in ['bottom','left']:
        ax.spines[axis].set_linewidth(1.5)
    ax.tick_params(width=1.5)
    fig.savefig("../Figures/all/TPR_%s_%s.pdf"%(rewireMode,networkName))
    plt.close(fig)





fig = plt.figure(figsize=(3*1.61803398875,3))
ax = plt.axes((0.2, 0.2, 0.70, 0.70), facecolor='w')
socfbNodeCounts = []
socfbAverageDegrees = []
socfbTPRAreas = []
for networkName,data in networkProperties.items():
    #only network names starting with socfb are interesting
    if(not (networkName.startswith("socfb") and ("TPR Data" in networkProperties[networkName]))):
        print("Skipping %s ..."%networkName)
        continue
    if(networkName=="socfb-nips-ego"): #ego network
        continue
    socfbNodeCounts.append(data["Node count"])
    socfbAverageDegrees.append(data["Average degree"])
    socfbTPRAreas.append(data["TPR Area"])
    trivialRatios = data["TPR Data"]
    
    ax.plot(alphaValues,trivialRatios,color = "#262626",lw=2.0,alpha=0.3)
    ax.fill_between(alphaValues,trivialRatios,1,color = "#E8EAEA",alpha=0.3,ec="face",lw=2.0)
ax.set_xlabel("$\\alpha$")
ax.set_ylabel("TPR")
ax.set_title("All socfb")
ax.set_xlim(-0.00,1.02)
ax.set_ylim(-0.020,1.020)

# create a list with two empty handles (or more if needed)
handles = [mpl_patches.Rectangle((0, 0), 1, 1, fc="white", ec="white", 
                                lw=0, alpha=0)] * 3

# create the corresponding number of labels (= the text you want to display)
labels = []
labels.append("$N$ = %d $\\pm$ %d"%(np.mean(socfbNodeCounts),np.std(socfbNodeCounts)))
labels.append("$\\langle k\\rangle$ = %.2f $\\pm$ %.2f"%(np.mean(socfbAverageDegrees),np.std(socfbAverageDegrees)))
labels.append("$Area_{\\mathrm{TPR}}$  %.2f $\\pm$ %.2f"%(np.mean(socfbTPRAreas),np.std(socfbTPRAreas)))

    # create the legend, supressing the blank space of the empty line symbol and the
    # padding between symbol and label by setting handlelenght and handletextpad
ax.legend(handles, labels, loc='best', 
        fancybox=False, framealpha=0, 
        handlelength=0, handletextpad=0)

ax.spines['right'].set_visible(False)
ax.spines['top'].set_visible(False)
for axis in ['bottom','left']:
    ax.spines[axis].set_linewidth(1.5)
ax.tick_params(width=1.5)
fig.savefig("../Figures/all/TPR_%s_All_socfb.pdf"%(rewireMode))
plt.close(fig)



import matplotlib.patches as mpl_patches

allCategoriesData = {
 'Benchmarks_t1-2.00_t2-1.00':{'name': "LFR", "data":[]},
 'Biological':{'name': "Biological", "data":[]},
 'GN':{'name': "SBM", "data":[]},
 'Information/Economy':{'name': "Information_Economy", "data":[]},
 'Models':{'name': "Models", "data":[]},
 'Social':{'name': "Social", "data":[]},
 'Transport-Geographic':{'name': "Transport-Geographic", "data":[]},
 }
allCategoriesData["Economy"] = allCategoriesData["Information/Economy"]
allCategoriesData["Information"] = allCategoriesData["Information/Economy"]
import math

networkPropertiesByCategory = {}
for networkName,data in networkProperties.items():
    #Remove Hybrid R = 0.001 N = 1000   
    if(networkName=="hybrid-r-0.001-n-1000"):
        continue;
    if("TPR Data" in data):
        allCategoriesData[data["Category"]]["data"].append(networkName)

borders = 0.75
spacing = 0.75
scale = 1.5
goldenRatio = (1+math.sqrt(5))/2
columnsCount = 6

for category,categoryData in allCategoriesData.items():

    networkNames = categoryData["data"]
    networkCount = len(networkNames)
    rowsCount = math.ceil(networkCount/columnsCount)
    totalWidth = scale*goldenRatio*columnsCount+spacing*(columnsCount-1)+borders*2
    totalHeight = scale*rowsCount+spacing*(rowsCount-1)+borders*2
    fig,axes = plt.subplots(rowsCount,columnsCount,
        figsize=(
            totalWidth,
            totalHeight
            ),
    sharey="row",sharex="col",
    gridspec_kw={
    "wspace":spacing/totalWidth*(columnsCount-1),
    "hspace":spacing/totalHeight*(rowsCount-1),
    "left":borders/totalWidth,
    "right":1.0-borders/totalWidth,
    "top":1.0-borders/totalHeight,
    "bottom":borders/totalHeight,
    })
    tableData = {
        "Network":[],
        "N":[],
        "$\\langle k\\rangle$":[],
        "$Q_r$":[],
    }
    for i in range(columnsCount*rowsCount):
        ax = axes[int(i/columnsCount),i%columnsCount]
        ax.axis('off')
    for networkIndex,networkName in enumerate(networkNames):
        data = networkProperties[networkName]
        if("TPR Data" not in data):
            print("Skipping %s ..."%networkName)
            continue
        # ax = plt.axes((0.2, 0.2, 0.70, 0.70), facecolor='w')
        rowIndex = int(networkIndex/columnsCount)
        columnIndex = networkIndex%columnsCount
        nextNetworkIndex = networkIndex+columnsCount
        ax = axes[rowIndex,columnIndex]
        ax.axis('on')
        nodeCount = data["Node count"]
        averageDegree = data["Average degree"]
        TPRArea = data["TPR Area"]
        trivialRatios = data["TPR Data"]
        ax.plot(alphaValues,trivialRatios,color = "#262626",lw=2.0)
        ax.fill_between(alphaValues,trivialRatios,1,color = "#E8EAEA")
        if(rowIndex==(rowsCount-1) or nextNetworkIndex>=networkCount):
            ax.set_xlabel("$\\alpha$")
            ax.tick_params(labelbottom=True)
        if(columnIndex==0):
            ax.set_ylabel("TPR")
        ax.set_title(data["Label"])
        ax.set_xlim(-0.00,1.02)
        ax.set_ylim(-0.020,1.020)
        # create a list with two empty handles (or more if needed)
        handles = [mpl_patches.Rectangle((0, 0), 1, 1, fc="white", ec="white", 
                                        lw=0, alpha=0)] * 3

        # create the corresponding number of labels (= the text you want to display)
        labels = []
        labels.append("$N$ = %d"%nodeCount)
        labels.append("$\\langle k\\rangle$ = %.2f"%averageDegree)
        labels.append("$Q_{r}$ = %.2f"%TPRArea)
        tableData["Network"].append(data["Label"])
        tableData["N"].append(nodeCount)
        tableData["$\\langle k\\rangle$"].append(averageDegree)
        tableData["$Q_r$"].append(TPRArea)
        # create the legend, supressing the blank space of the empty line symbol and the
        # padding between symbol and label by setting handlelenght and handletextpad
        ax.legend(handles, labels, loc='best', 
                fancybox=False, framealpha=0, 
                handlelength=0, handletextpad=0)
        ax.spines['right'].set_visible(False)
        ax.spines['top'].set_visible(False)
        for axis in ['bottom','left']:
            ax.spines[axis].set_linewidth(1.5)
        ax.tick_params(width=1.5)
    pd.DataFrame(tableData) \
    .sort_values("$Q_r$",ascending=False) \
    .to_latex("../Figures/table_TPR_byCategory_%s_%s.tex"%(rewireMode,categoryData["name"]), \
    index=False,
    formatters={
        "$Q_r$": lambda value:"%.2f"%value,
        "$\\langle k\\rangle$": lambda value:"%.1f"%value,
        "N": lambda value:"%d"%value,
    })
    fig.savefig("../Figures/TPR_byCategory_%s_%s.pdf"%(rewireMode,categoryData["name"]))
    plt.close(fig)






import louvain
import igraph as ig
from graph_tool import openmp_set_num_threads as gtOpenmp_set_num_threads
from graph_tool import Graph as gtGraph
import graph_tool.inference as gtInference
import numpy as np
from tqdm.auto import tqdm, trange
from collections import Counter
import multiprocessing as mp
import louvain
import os

def LouvainModularity(aNetwork):
    partition = louvain.find_partition(
        aNetwork, louvain.ModularityVertexPartition)
    return partition.quality()


def SBMMinimizeMembership(vertexCount, edges, directed=False, degreeCorrected=True):
    g = gtGraph(directed=directed)
    for _ in range(0, vertexCount):
        g.add_vertex()
    for edge in edges:
        g.add_edge(edge[0], edge[1])
    state = gtInference.minimize.minimize_blockmodel_dl(
        g, state_args={"deg_corr": degreeCorrected})
    DLDetected = state.entropy()
    DLTrivial = gtInference.blockmodel.BlockState(
        g, B=1, deg_corr=degreeCorrected).entropy()
    return (list(state.get_blocks()), DLDetected, DLTrivial)


def calculateMaxModularity(g, trials=100):
    maxModularity = -1
    for _ in range(trials):
        modularity = LouvainModularity(g)
        if(modularity > maxModularity):
            maxModularity = modularity
    return maxModularity


def rewireNetwork(nodeCount, edges, probability):
    edgesCount = len(edges)
    newEdges = np.array(edges)
    mask = np.random.random(edgesCount) < probability
    selectedEdgesIndices = np.where(mask)[0]
    generatedSelectedEdges = np.array(np.random.randint(
        0, nodeCount, (len(selectedEdgesIndices), 2)))
    newEdges[selectedEdgesIndices] = generatedSelectedEdges
    newEdges = [(fromIndex, toIndex) for fromIndex, toIndex in newEdges]
    return newEdges


def getMajorConnectedComponent(nodeCount, edges, directed=False):
    g = ig.Graph(nodeCount, edges, directed=directed).simplify()
    giant = g.components(mode="weak").giant()
    return (giant.vcount(), giant.get_edgelist())


def calculatePerturbedTrivialCount(args):
    #Reseeding
    np.random.seed(int.from_bytes(os.urandom(4), byteorder='little'))
    trivialCount = 0
    nodeCount, edges, directed, probability, \
        detectionTrials = args
    newEdges = rewireNetwork(nodeCount, edges, probability)
    newNodeCount = nodeCount
    allDLDetected = []
    allDLTrivial = []
    (newNodeCount, newEdges) = getMajorConnectedComponent(
        nodeCount, newEdges, directed)
    for detectionIndex in range(0, detectionTrials):
        communities, DLDetected, DLTrivial = SBMMinimizeMembership(
            newNodeCount, newEdges, directed=directed)
        if(len(set(communities)) == 1):
            trivialCount += 1
        allDLDetected.append(DLDetected)
        allDLTrivial.append(DLTrivial)
    return trivialCount, allDLDetected, allDLTrivial


def RModularity(
    nodeCount,
    edges,
    directed=False,
    perturbationCount=25,
    detectionTrials=1,
    rewireResolution=51,
    outputCurves=False,
    showProgress=True,
    useMultiprocessing=True
):
    """
    Computes the Robustness Modularity of a network.

    Parameters
    ----------
    nodeCount : int
        The number of nodes in the network.
    edges : list of tuples
        A list of the edges in the network.
    directed : int, optional
        Whether the network is directed or not.
    perturbationCount : int, optional
        The number of perturbations to perform.
        (defaults to 25)
    detectionTrials : int, optional
        The number of times to perform community
        detection for each perturbed network.
        (defaults to 1)
    rewireResolution : int, optional
        The number values points for the rewire
        probabilities (from 0 to 1) to calculate
        the Trivial Partition Ratio (TPR) curves
        and Robustness Modularity.
        (defaults to 51)
    outputCurves : bool, optional
        Whether to save the TPR and DL curves.
        (defaults to False)
    showProgress : bool, optional
        Shows a progress bar if enabled.
        (defaults to True)
    useMultiprocessing: bool, optional
        Uses parallel processing to calculate
        Rmodularity.
        (defaults to True)
    Returns
    -------
    float 
        The RModularity of the network.
    (float, np.array dim=1, np.array dim=1, np.array dim=2, np.array dim=2) if outputCurves is True
        The tuple (RModularity, probabilities, TPR curves, DL Detected, DL Trivial) containing
        the Robustness Modularity, the rewire probabilities, the TPR curves, the Description
        lenghts for the detected and trivial partitions.
    """
    TPRCurve = np.zeros(rewireResolution)
    DLCurvesDetected = np.zeros(
        (rewireResolution, detectionTrials*perturbationCount))
    DLCurvesTrivial = np.zeros(
        (rewireResolution, detectionTrials*perturbationCount))
    probabilities = np.linspace(0, 1, rewireResolution)

    (nodeCount, edges) = getMajorConnectedComponent(
        nodeCount, edges, directed)
    if(showProgress):
        probabilitiesIterator = tqdm(probabilities, desc="Current p")
    else:
        probabilitiesIterator = probabilities
    if(useMultiprocessing):
        num_processors = mp.cpu_count()
        # Disabling internal multithreading of graph_tool
        gtOpenmp_set_num_threads(1)

    for probabilityIndex, probability in enumerate(probabilitiesIterator):
        trivialCount = 0
        if(useMultiprocessing):
            allArgs = [(nodeCount, edges, directed, probability,
                        detectionTrials)]*perturbationCount
            perturbationIndex = 0

            pool = mp.Pool(processes=num_processors)
            for newTrivialCount, allDLDetected, allDLTrivial in tqdm(pool.imap_unordered(func=calculatePerturbedTrivialCount, iterable=allArgs), total=len(allArgs), desc="Perturbation", leave=False):
                trivialCount += newTrivialCount
                DLCurvesTrivial[probabilityIndex, perturbationIndex *
                                detectionTrials:(perturbationIndex+1)*detectionTrials] = allDLTrivial
                DLCurvesDetected[probabilityIndex, perturbationIndex *
                                 detectionTrials:(perturbationIndex+1)*detectionTrials] = allDLDetected
                perturbationIndex += 1
            pool.terminate()
            pool.close()
            TPRCurve[probabilityIndex] = trivialCount / \
                (perturbationCount*detectionTrials)
        else:
            for perturbationIndex in trange(0, perturbationCount, desc="Perturbation", leave=False):
                args = (nodeCount, edges, directed,
                        probability, detectionTrials)
                newTrivialCount, allDLDetected, allDLTrivial = calculatePerturbedTrivialCount(
                    args)
                trivialCount += newTrivialCount
                DLCurvesTrivial[probabilityIndex, perturbationIndex *
                                detectionTrials:(perturbationIndex+1)*detectionTrials] = allDLTrivial
                DLCurvesDetected[probabilityIndex, perturbationIndex *
                                 detectionTrials:(perturbationIndex+1)*detectionTrials] = allDLDetected
                perturbationIndex += 1
            TPRCurve[probabilityIndex] = trivialCount / \
                (perturbationCount*detectionTrials)

    RModularity = 1.0-np.trapz(TPRCurve, probabilities)

    if(outputCurves):
        return (RModularity, probabilities, TPRCurve, DLCurvesTrivial, DLCurvesDetected)
    else:
        return RModularity


def modularityNullmodel(args):
    network, detectionTrials = args
    networkConfig = ig.Graph.Degree_Sequence(
        network.degree()).simplify().components(mode="weak").giant()
    return calculateMaxModularity(networkConfig, trials=detectionTrials)


def modularityDifference(
    nodeCount,
    edges,
    directed=False,
    detectionTrials=100,
    nullmodelCount=100,
    detectionTrialsNullModel=10,
    showProgress=True,
    useMultiprocessing=True
):
    """
        Computes the Modularity Difference of a network.

    Parameters
    ----------
    nodeCount : int
        The number of nodes in the network.
    edges : list of tuples
        A list of the edges in the network.
    directed : int, optional
        Whether the network is directed or not.
    detectionTrials : int, optional
        The number of times to perform community
        detection for each perturbed network.
        (defaults to 10)
    nullmodelCount : int, optional
        The number of times to perform community
        detection using nullmodels.
        (defaults to 200)
    detectionTrialsNullModel : int, optional
        The number of times to perform community
        detection for each nullmodel realization.
        (defaults to 200)
    showProgress : bool, optional
        Shows a progress bar if enabled.
        (defaults to True)
    Returns
    -------
    float 
        The Modularity Difference of the network.
    """
    network = ig.Graph(nodeCount, edges, directed=directed).simplify(
    ).components(mode="weak").giant()
    modularity = calculateMaxModularity(network, trials=detectionTrials)
    if(useMultiprocessing):
        num_processors = mp.cpu_count()
    nullModelModularities = []
    if(useMultiprocessing):
        allArgs = [(network, detectionTrials)]*nullmodelCount
        pool = mp.Pool(processes=num_processors)
        for nullModelModularity in tqdm(pool.imap_unordered(func=modularityNullmodel, iterable=allArgs), total=len(allArgs), desc="NullModel"):
            nullModelModularities.append(nullModelModularity)
        pool.terminate()
        pool.close()
    else:
        nullModelIterator = range(nullmodelCount)
        if(showProgress):
            nullModelIterator = tqdm(nullModelIterator, desc="Nullmodel")
        for _ in nullModelIterator:
            nullModelModularity = modularityNullmodel(
                (network, detectionTrials))
            nullModelModularities.append(nullModelModularity)
    modularityDifference = modularity - np.mean(nullModelModularities)
    return modularityDifference


def informationModularity(
        nodeCount,
        edges,
        directed=False):
    """
        Computes the Information Modularity of a network.

    Parameters
    ----------
    nodeCount : int
        The number of nodes in the network.
    edges : list of tuples
        A list of the edges in the network.
    directed : int, optional
        Whether the network is directed or not.
    Returns
    -------
    float 
        The Information Modularity of the network.
    """
    _, DLDetected, DLTrivial = SBMMinimizeMembership(
        nodeCount, edges, directed)
    return 1-(DLDetected/DLTrivial)




import benchmarkParameters as bp
import utilities as ut
import importlib
import numpy as np
from multiprocessing import Process, Manager, Pool
import multiprocessing
import json
from tqdm.auto import tqdm
import pandas as pd
importlib.reload(ut)
import matplotlib.gridspec as gridspec
from adjustText import adjust_text;
import matplotlib
#Disable Matplotlib interface
matplotlib.use('Agg')
import matplotlib.pyplot as plt

plt.rc('font', family='sans-serif')
# plt.rc('text', usetex=True)
plt.rc('xtick', labelsize=15)
plt.rc('ytick', labelsize=15)
plt.rc('axes', labelsize=16)
plt.rc('legend', fontsize=14)

variant = "SBMBasic"
rewireMode = "basic"
alphaPoints = 51
alphaValues = np.linspace(0,1.0,alphaPoints);
networkList = ut.getNetworkList()
trivialThreshold = 1.0

majorSizesByNetwork = {}

with open("../Data/majorSizes_%s.json"%rewireMode,"rt") as fd:
    majorSizesByNetwork = json.load(fd)

networkList = [(networkName,label,category) for networkName,label,category in networkList if networkName not in majorSizesByNetwork]

num_processors = multiprocessing.cpu_count()

# print('will use processors = ', str(num_processors))

# print('starting at', dt_string)
def processForAlpha(parameters):
    alpha,networkName = parameters
    majorForAlpha = []
    try:
        for perturbationIndex in range(bp.perturbationRealizations):
            g = ut.getPerturbedNetwork(networkName,rewireMode,alpha,perturbationIndex)
            majorForAlpha.append(g.clusters(mode="WEAK").giant().vcount())
            # data = ut.loadDistributionData("NC","%s_%s"%(rewireMode,variant),networkName)
            # data = ut.loadDistributionData("NC","%s_%s"%(rewireMode,variant),networkName)
            # data = ut.loadDistributionData("NC","%s_%s"%(rewireMode,variant),networkName)
    except OSError:
        print("Problem with %s"%networkName)
    return majorForAlpha

pool = Pool(processes=num_processors)

for entry in tqdm(networkList,leave=False):
    networkName,label,category  =  entry
    majorSizes = []
    majorSizesByNetwork[networkName] = majorSizes
    for majorForAlpha in tqdm(pool.imap(func=processForAlpha, iterable=[(alpha,networkName) for alpha in alphaValues]), total=len(alphaValues),leave=False):
        if(majorForAlpha):
            majorSizes.append(majorForAlpha)
        else:
            del majorSizesByNetwork[networkName]
            break
    with open("../Data/majorSizes_%s_NEW.json"%rewireMode,"wt") as fd:
        json.dump(majorSizesByNetwork,fd)


with open("../Data/majorSizes_%s.json"%rewireMode,"wt") as fd:
    json.dump(majorSizesByNetwork,fd)


pool.close()
pool.terminate()






import numpy as np
import math;
import benchmarkParameters as bp
import createGNNetworks as gn
import sys
import scipy
import scipy.signal
import os
import adjustText as at
import matplotlib.gridspec as gridspec
from adjustText import adjust_text;
from collections import OrderedDict
import matplotlib
#Disable Matplotlib interface
matplotlib.use('Agg')

import matplotlib.pyplot as plt

plt.rc('font', family='sans-serif')
# plt.rc('text', usetex=True)
plt.rc('xtick', labelsize=15)
plt.rc('ytick', labelsize=15)
plt.rc('axes', labelsize=16)
plt.rc('legend', fontsize=14)

lineStyles = ['-' , '--' , '-.' , ':','-' , '--' , '-.' , ':'];
lineWidths = [3,3,3,3,1,1,1,1];

styles = [
	("#1f77b4","-"),
	("#ff7f0e","-"),
	("#2ca02c","-"),
	("#d62728","-"),
	("#9467bd","-"),
	("#8c564b","-"),
	("#e377c2","-"),
	("#7f7f7f","-"),
	("#bcbd22","-"),
	("#17becf","-"),
	("#aec7e8","-"),
	("#ffbb78","--"),
	("#98df8a","--"),
	("#ff9896","--"),
	("#c5b0d5","--"),
	("#c49c94","--"),
	("#f7b6d2","--"),
	("#c7c7c7","--"),
	("#dbdb8d","--"),
	("#9edae5","--"),
	("#1f77b4","-."),
	("#ff7f0e","-."),
	("#2ca02c","-."),
	("#d62728","-."),
	("#9467bd","-."),
	("#8c564b","-."),
	("#e377c2","-."),
	("#7f7f7f","-."),
	("#bcbd22","-."),
	("#17becf","-."),
	("#aec7e8","-."),
	("#ffbb78",":"),
	("#98df8a",":"),
	("#ff9896",":"),
	("#c5b0d5",":"),
	("#c49c94",":"),
	("#f7b6d2",":"),
	("#c7c7c7",":"),
	("#dbdb8d",":"),
	("#9edae5",":"),
	("#1f77b4","-"),
	("#ff7f0e","-"),
	("#2ca02c","-"),
	("#d62728","-"),
	("#9467bd","-"),
	("#8c564b","-"),
	("#e377c2","-"),
	("#7f7f7f","-"),
	("#bcbd22","-"),
	("#17becf","-"),
	("#aec7e8","-"),
	("#9467bd","-"),
	("#8c564b","-"),
	("#e377c2","-"),
	("#7f7f7f","-"),
	("#bcbd22","-"),
	("#17becf","-"),
	("#aec7e8","-"),
	("#ffbb78",":"),
	("#98df8a",":"),
	("#ff9896",":"),
	("#c5b0d5",":"),
	("#c49c94",":"),
	("#f7b6d2",":"),
	("#c7c7c7",":"),
	("#dbdb8d",":"),
	("#9edae5",":"),
	("#1f77b4","-"),
	("#ff7f0e","-"),
	("#2ca02c","-"),
	("#d62728","-"),
	("#9467bd","-"),
	("#8c564b","-"),
	("#e377c2","-"),
	("#7f7f7f","-"),
	("#bcbd22","-"),
	("#17becf","-"),
	("#aec7e8","-"),
	("#9467bd","-"),
	("#8c564b","-"),
	("#e377c2","-"),
	("#7f7f7f","-"),
	("#bcbd22","-"),
	("#17becf","-"),
	("#aec7e8","-"),
];

def plotData(ax,measurement,networkName,label,plotIndex):
	try:
		dataArray = np.loadtxt("../Data/Results/pairwise%s_%s_%s.txt"%(measurement,variant,networkName));
	except:
		print("!NOT FOUND: %s"%("../Data/Results/pairwise%s_%s_%s.txt"%(measurement,variant,networkName)));
		return;
	print("FOUND %s"%networkName);
	if(len(dataArray)==0):
		return;
	values = dataArray[:,0];
	errors = dataArray[:,1];
	alphaValues = np.linspace(0,1.0,len(values));
	currentStyle = styles[plotIndex%len(styles)];
	ax.plot(alphaValues,values,color=currentStyle[0],linestyle=currentStyle[1],label=label);
	ax.fill_between(alphaValues,values-errors,values+errors,color=currentStyle[0],alpha=0.2);
	




realNetworkNames = [
	("Airports", "Airports","Transport-Geographic"),
	("BA", "BA","Models"),
	("BA5000k18-P1", "BA k=18 p=1","Models"),
#	("BA5000k18-P2", "BA k=18 p=2","Models"),
#	("BA5000k18-P3", "BA k=18 p=3","Models"),
	("ER", "ER","Models"),
	("GEO", "GEO","Models"),
	("Oldenburg", "Oldenburg","Transport-Geographic"),
	("RVOR", "RVOR","Models"),
	("San Joaquin", "San Joaquin","Transport-Geographic"),
	("VOR", "VOR","Models"),
	("WAX", "WAX","Models"),
	("Wikipedia", "Wikipedia","Information"),
	("CollegeMsg-major", "CollegeMsg-major","Social"),
	("CA-HepTh", "CA-HepTh","Social"),
	("Cit-HepTh", "Cit-HepTh","Information"),
	("Email-Enron", "Email-Enron","Social"),
	("email-Eu-core", "email-Eu-core","Social"),
	("Wiki-Vote-major", "Wiki-Vote-major","Information"),
	("facebook_combined", "Facebook_combined","Social"),
	("soc-sign-bitcoinotc_positive", "soc-sign-bitcoinotc_positive","Social"),
	("CA-GrQc", "CA-GrQc","Social"),
# 	("robertson_1929", "robertson_1929","Social"),
	("USairport_2010", "USairport_2010","Transport-Geographic"),
	# ("kasthuri_graph_v4", "kasthuri_graph_v4"),
	("drosophila_medulla_1", "drosophila_medulla_1","Biological"),
	# ("net2m_2011-05-01", "net2m_2011-05-01"),
	# ("net2m_2010-10-01", "net2m_2010-10-01"),
	# ("net2m_2010-04-01", "net2m_2010-04-01"),
	# ("net2m_2009-06-01", "net2m_2009-06-01"),
	# ("net2m_2008-01-01", "net2m_2008-01-01"),
	# ("net2m_2002-12-01", "net2m_2002-12-01"),
	("roget", "roget","Information"),
	("quakers", "quakers","Social"),
	("LC_multiple", "LC_multiple","Biological"),
	("WesternUS_PowerGrid", "WesternUS_PowerGrid","Transport-Geographic"),
	("Collins", "Collins","Biological"),
	("wikispeedia_paths-and-graph", "wikispeedia_paths-and-graph","Information"),
# 	("soc-sign-bitcoinalpha_negative", "soc-sign-bitcoinalpha_negative","Social"),
	("soc-sign-bitcoinalpha_positive", "soc-sign-bitcoinalpha_positive","Social"),
# 	("soc-sign-bitcoinotc_negative", "soc-sign-bitcoinotc_negative","Social"),
	("CA-HepPh", "CA-HepPh","Social"),
	("ScientometricsAndJInformetrics","ScientometricsAndJInformetrics","Information"),
	("dep_2010_obstr_0.8_leidenalg_dist","dep_2010_obstr_0.8_leidenalg_dist","Social"),
	("wiki_PortugalandBrazil","wiki_PortugalandBrazil","Information"),
	("wiki_MarvelvsDC","wiki_MarvelvsDC","Information"),
	("email","email","Social"),
	("netscience","netscience","Information"),
	("mouse_retina_1","mouse_retina_1","Biological"),
	# ("autobahn","autobahn","Transport-Geographic"),
	("OClinks_w","OClinks_w","Social"),
	("AI_interactions","AI_interactions","Biological"),
	("OF_one-mode_weightedchar_Newman","OF_one-mode_weightedchar_Newman","Social"),
	("CA-heaberlin_dedeo_norm_network","CA-heaberlin_dedeo_norm_network","Biological"),
	("inf-euroroad","inf-euroroad","Transport-Geographic"),
	("road-euroroad","road-euroroad","Transport-Geographic"),
	("ia-crime-moreno","ia-crime-moreno","Information"),
	("rt_voteonedirection","rt_voteonedirection","Social"),
	("bn-mouse-kasthuri_graph_v4","bn-mouse-kasthuri_graph_v4","Biological"),
	("power-1138-bus","power-1138-bus","Transport-Geographic"),
	("bio-CE-GT","bio-CE-GT","Biological"),
	("rt_assad","rt_assad","Social"),
	("bio-CE-HT","bio-CE-HT","Biological"),
	("rt_occupywallstnyc","rt_occupywallstnyc","Social"),
	("rt_obama","rt_obama","Social"),
	("mammalia-voles-plj-trapping","mammalia-voles-plj-trapping","Biological"),
	("mammalia-voles-kcs-trapping","mammalia-voles-kcs-trapping","Biological"),
	("rt_damascus","rt_damascus","Social"),
	("rt_occupy","rt_occupy","Social"),
	("socfb-nips-ego","socfb-nips-ego","Social"),
	("rt_israel","rt_israel","Social"),
	("mammalia-voles-rob-trapping","mammalia-voles-rob-trapping","Biological"),
	("bio-grid-mouse","bio-grid-mouse","Biological"),
	("rt_islam","rt_islam","Social"),
	("rt_tlot","rt_tlot","Social"),
	("rt_lebanon","rt_lebanon","Social"),
	("bio-DM-HT","bio-DM-HT","Biological"),
	("mammalia-voles-bhp-trapping","mammalia-voles-bhp-trapping","Biological"),
	("rt_tcot","rt_tcot","Social"),
	("rt_gop","rt_gop","Social"),
	("power-bcspwr09","power-bcspwr09","Transport-Geographic"),
	("rt_libya","rt_libya","Social"),
	("bio-yeast-protein-inter","bio-yeast-protein-inter","Biological"),
	("rt_oman","rt_oman","Social"),
	("rt_p2","rt_p2","Social"),
	("rt_uae","rt_uae","Social"),
	("econ-mahindas","econ-mahindas","Economy"),
	("rt_gmanews","rt_gmanews","Social"),
	("rt_alwefaq","rt_alwefaq","Social"),
	("ia-email-univ","ia-email-univ","Social"),
	("rt_dash","rt_dash","Social"),
	("rt_onedirection","rt_onedirection","Social"),
	("rt_mittromney","rt_mittromney","Social"),
	("rt_barackobama","rt_barackobama","Social"),
	("econ-poli","econ-poli","Economy"),
	("rt_bahrain","rt_bahrain","Social"),
	("ia-fb-messages","ia-fb-messages","Social"),
	("bio-grid-plant","bio-grid-plant","Biological"),
	("rt_saudi","rt_saudi","Social"),
	("rt_ksa","rt_ksa","Social"),
	("tech-routers-rf","tech-routers-rf","Information"),
	("web-edu","web-edu","Information"),
	("ia-dnc-corecipient","ia-dnc-corecipient","Social"),
	("rt_qatif","rt_qatif","Social"),
	("rt_lolgop","rt_lolgop","Social"),
	("rt_justinbieber","rt_justinbieber","Social"),
	("inf-power","inf-power","Transport-Geographic"),
	("power-US-Grid","power-US-Grid","Transport-Geographic"),
	("ca-Erdos992","ca-Erdos992","Social"),
	("rt_http","rt_http","Social"),
	("ia-reality","ia-reality","Social"),
	("bio-HS-HT","bio-HS-HT","Biological"),
	("web-EPA","web-EPA","Information"),
	("power-eris1176","power-eris1176","Transport-Geographic"),
	("bio-grid-worm","bio-grid-worm","Biological"),
	("bio-SC-LC","bio-SC-LC","Biological"),
	("power-bcspwr10","power-bcspwr10","Transport-Geographic"),
	("soc-hamsterster","soc-hamsterster","Social"),
	("fb-pages-tvshow","fb-pages-tvshow","Social"),
	("bio-SC-GT","bio-SC-GT","Biological"),
	("bio-SC-CC","bio-SC-CC","Biological"),
	("bio-grid-fission-yeast","bio-grid-fission-yeast","Biological"),
	("tech-pgp","tech-pgp","Information"),
	("bio-HS-LC","bio-HS-LC","Biological"),
	("bio-dmela","bio-dmela","Biological"),
	("bio-CE-PG","bio-CE-PG","Biological"),
	("inf-openflights","inf-openflights","Transport-Geographic"),
	("socfb-Simmons81","socfb-Simmons81","Social"),
	("bio-CE-GN","bio-CE-GN","Biological"),
	("soc-advogato","soc-advogato","Social"),
	("web-spam","web-spam","Information"),
# 	("ia-escorts-dynamic","ia-escorts-dynamic","Social"), Bipartide
	("bio-SC-HT","bio-SC-HT","Biological"),
	("fb-pages-politician","fb-pages-politician","Social"),
	("bio-grid-fruitfly","bio-grid-fruitfly","Biological"),
	("web-indochina-2004","web-indochina-2004","Information"),
	("bio-DM-CX","bio-DM-CX","Biological"),
	("socfb-Haverford76","socfb-Haverford76","Social"),
	("tech-WHOIS","tech-WHOIS","Information"),
	("bio-DR-CX","bio-DR-CX","Biological"),
	("socfb-Swarthmore42","socfb-Swarthmore42","Social"),
	("econ-orani678","econ-orani678","Economy"),
	("bio-grid-human","bio-grid-human","Biological"),
	("socfb-USFCA72","socfb-USFCA72","Social"),
	("rec-movielens-user-movies-10m","rec-movielens-user-movies-10m","Information"),
	("fb-pages-public-figure","fb-pages-public-figure","Social"),
	("bio-HS-CX","bio-HS-CX","Biological"),
	("soc-wiki-elec","soc-wiki-elec","Social"),
	("bio-WormNet-v3-benchmark","bio-WormNet-v3-benchmark","Biological"),
	("socfb-Mich67","socfb-Mich67","Social"),
	("socfb-Bowdoin47","socfb-Bowdoin47","Social"),
	("socfb-Amherst41","socfb-Amherst41","Social"),
	("socfb-Oberlin44","socfb-Oberlin44","Social"),
	("fb-pages-government","fb-pages-government","Social"),
	("socfb-Wellesley22","socfb-Wellesley22","Social"),
	("socfb-Hamilton46","socfb-Hamilton46","Social"),
	("socfb-Smith60","socfb-Smith60","Social"),
	("socfb-Trinity100","socfb-Trinity100","Social"),
	("socfb-Williams40","socfb-Williams40","Social"),
	("socfb-Vassar85","socfb-Vassar85","Social"),
	("socfb-Middlebury45","socfb-Middlebury45","Social"),
	("socfb-Brandeis99","socfb-Brandeis99","Social"),
	("socfb-Wesleyan43","socfb-Wesleyan43","Social"),
	("socfb-Pepperdine86","socfb-Pepperdine86","Social"),
	("socfb-Santa74","socfb-Santa74","Social"),
	("socfb-Colgate88","socfb-Colgate88","Social"),
	("socfb-Bucknell39","socfb-Bucknell39","Social"),
	("socfb-UC64","socfb-UC64","Social"),
	("socfb-Rochester38","socfb-Rochester38","Social"),
	("bio-CE-CX","bio-CE-CX","Biological"),
	("socfb-Rice31","socfb-Rice31","Social"),
	("socfb-JohnsHopkins55","socfb-JohnsHopkins55","Social"),
	("socfb-Vermont70","socfb-Vermont70","Social"),
	("socfb-Lehigh96","socfb-Lehigh96","Social"),
	("socfb-Howard90","socfb-Howard90","Social"),
	("socfb-UChicago30","socfb-UChicago30","Social"),
	("socfb-American75","socfb-American75","Social"),
	("socfb-UCSC68","socfb-UCSC68","Social"),
	("socfb-Maine59","socfb-Maine59","Social"),
	("socfb-CMU","socfb-CMU","Social"),
	("socfb-Carnegie49","socfb-Carnegie49","Social"),
	("socfb-MIT","socfb-MIT","Social"),
	("socfb-MIT8","socfb-MIT8","Social"),
	("socfb-Tufts18","socfb-Tufts18","Social"),
	("socfb-William77","socfb-William77","Social"),
	("socfb-Wake73","socfb-Wake73","Social"),
	("socfb-Tulane29","socfb-Tulane29","Social"),
	("socfb-Princeton12","socfb-Princeton12","Social"),
	("socfb-Dartmouth6","socfb-Dartmouth6","Social"),
	("bio-grid-yeast","bio-grid-yeast","Biological"),
	("socfb-Villanova62","socfb-Villanova62","Social"),
	("socfb-Emory27","socfb-Emory27","Social"),
	("econ-psmigr2","econ-psmigr2","Economy"),
	("econ-psmigr1","econ-psmigr1","Economy"),
	("econ-psmigr3","econ-psmigr3","Economy"),
	("socfb-Bingham82","socfb-Bingham82","Social"),
	("socfb-WashU32","socfb-WashU32","Social"),
	("socfb-Brown11","socfb-Brown11","Social"),
	("socfb-Yale4","socfb-Yale4","Social"),
	("scc_twitter-copen","scc_twitter-copen","Social"),
	("socfb-Georgetown15","socfb-Georgetown15","Social"),
	("socfb-Vanderbilt48","socfb-Vanderbilt48","Social"),
	("scc_fb-messages","scc_fb-messages","Social"),
	("socfb-Duke14","socfb-Duke14","Social"),
	("scc_reality","scc_reality","Social"),
	("baselineN50M100","baselineN50M100","Models"),
	("baselineN100M50","baselineN100M50","Models"),
	("baselineN3M2000","baselineN3M2000","Models"),
	("baselineN5M1000","baselineN5M1000","Models"),
	("baselineN10M500","baselineN10M500","Models"),
	("ERIgraph", "ERIgraph", "Models"),
	("ERIgraphLoops", "ERIgraphLoops", "Models"),
	("ERNXSparse", "ERNXSparse", "Models"),
	("ERMine", "ERMine", "Models"),
	("ERNXDense", "ERNXDense", "Models"),
	("ER", "ER", "Models"),
	("hybrid5000k20R0.500","Hybrid R = 0.500", "Models"),
	("hybrid5000k20R0.250","Hybrid R = 0.250", "Models"),
	("hybrid5000k20R0.125","Hybrid R = 0.125", "Models"),
	("hybrid1000k10R0.500","Hybrid N1000 R = 0.500", "Models"),
	("hybrid1000k10R0.250","Hybrid N1000 R = 0.250", "Models"),
	("hybrid1000k10R0.125","Hybrid N1000 R = 0.125", "Models"),
]

realNetworkNames = [entry for entry in realNetworkNames if not entry[0][0:2]=="rt"] 

plotName = "all";

benchmarkNames = [];
for networkRealization in range(bp.networkRealizations):
	for tau1 in bp.t1Values:
		for tau2 in bp.t2Values:
			for mu in bp.muValues:
				benchmarkNames.append((bp.getNetworkName(mu,networkRealization,tau1,tau2),"$\\mu=%.3f$"%mu,"Benchmarks_t1-%.2f_t2-%.2f"%(tau1,tau2)));
		
# for networkRealization in range(bp.networkRealizations):
# 	for mu in [0.55,0.62,0.64,0.66,0.68,0.75]:
# 		benchmarkNames.append((bp.getNetworkName(mu,networkRealization),"$\\mu=%.3f$"%mu,"Benchmarks"));

for networkRealization in range(bp.networkRealizations):
	for mu in gn.muValues:
		# benchmarkNames.append(gn.getNetworkName(gn.P_in,P_out,networkRealization));
		benchmarkNames.append((gn.getNetworkName(gn.nodesCount,gn.communitiesCount,gn.avgk,mu,networkRealization),"$\\mu=%.3f$"%mu,"GN"));


# for mu in bp.muValues:
# 	networkName = bp.getNetworkName(mu,networkRealization);
# 	plotData(ax,"NMI",networkName,"$\\mu=%.3f$"%mu,plotIndex);
#bp.getNetworkName(0,networkRealization))
networkNames = []
networkNames += benchmarkNames;#+realNetworkNames;
networkNames += realNetworkNames;
def loadDistributionData(measurement,variant,networkName,bins=100):
	data = [];
	try:
		with gzip.open("../Data/Results/pairwise%s_distrib_%s_%s.txt.gz"%(measurement,variant,networkName),"rt") as fd:
			for line in fd:
				entry = [float(value) for value in line.strip().split(" ")];
				data.append(entry); #np.histogram(entry,range=(0,1),bins=bins)[0]);
	except:
		print("!NOT FOUND: %s"%("../Data/Results/pairwise%s_distrib_%s_%s.txt.gz"%(measurement,variant,networkName)));
		return None;
	data = np.array(data);
	if(data.shape[0]==0):
		return None;
	else:
		return data;

figureSize = 3.0

rewireMode="basic";
variants = ["infomap","multilevel","multilevelZero","SBMBasic"," "];
variants = ["infomap","multilevel"," ","SBMBasic"," "];
variants = ["infomap","multilevel"," ","SBMBasic"," "];
variants = ["infomap","multilevel","SBMBasic"];
# variants = ["SBMBasic"];

import gzip
from matplotlib.colors import LogNorm

bins = 100

measurements = [
"NMI",
"AMI",
"NVI",
# "ARI",
# "VME",
"NC",
"CS",
"ACS",
"S",
"SR",
"DS",
]


legends = False;
networkRealization = 0;
plotIndex = 0;
columns = 4;
trivialThreshold = 1.0

allDataByCategory = {};
for measurement in measurements:
	for variant in variants:
	# 	variant="fast_%s"%variant;
		labels = [];
		for networkName,label,category in networkNames:
			if(category not in allDataByCategory):
				allDataByCategory[category] = OrderedDict();
			distribData = loadDistributionData(measurement,"%s_%s"%(rewireMode,variant),networkName);
			if(distribData is not None):
				if(networkName not in allDataByCategory[category]):
					allDataByCategory[category][networkName] = OrderedDict();
				if(variant not in allDataByCategory[category][networkName]):
					allDataByCategory[category][networkName][variant] = OrderedDict();
				if(measurement not in allDataByCategory[category][networkName][variant]):
					allDataByCategory[category][networkName][variant][measurement] = OrderedDict();
				allDataByCategory[category][networkName]["LABEL"] = label;
				allDataByCategory[category][networkName][variant][measurement] = distribData;


networkSizes = {};
for variant in variants:
	labels = [];
	for networkName,label,category in networkNames:
		if(category not in allDataByCategory):
			allDataByCategory[category] = OrderedDict();
		if(category not in allDataByCategory):
			continue;
		if(networkName not in allDataByCategory[category]):
			continue;
		if(variant not in allDataByCategory[category][networkName]):
			continue;
		networkData = allDataByCategory[category][networkName][variant];
		if("S" not in networkData):
			continue
		if("NC" not in networkData or networkData["NC"] is None or
			"CS" not in networkData or networkData["CS"] is None):
			continue;
		# print("OK %s"%networkName);
		averageCommunitySizes = networkData["ACS"];
		averageEntropy = networkData["S"]
		communitySizes = networkData["CS"];
		numberOfCommunities = networkData["NC"];
		verticesCount = int(np.sum(communitySizes[0])/numberOfCommunities.shape[1]);
		networkSizes[networkName] = verticesCount;
		majorCommunitySizes = [];
		errorMajorCommunitySizes = [];
		trivialRatios = [];
		for alphaIndex in range(len(numberOfCommunities)):
			communityIndices = np.cumsum(numberOfCommunities[alphaIndex]);
			majorCommunitySizesDistrib=[];
			errorMajorCommunitySizesDistrib=[];
			trivialCount = 0;
			for i in range(len(communityIndices)):
				if(i>0):
					startIndex = int(communityIndices[i-1]);
				else:
					startIndex=0;
				endIndex = int(communityIndices[i]);
				communitySizesDistrib = communitySizes[alphaIndex][startIndex:endIndex];
				avgSize = np.average(communitySizesDistrib);
				avgDiff = abs(avgSize - averageCommunitySizes[alphaIndex][i]);
				if(avgDiff>=1):
					print("MAJOR ERROR!!!!!!!! %f %f"%(avgSize,averageCommunitySizes[alphaIndex][i]));
				majorCommunitySizesDistrib.append(np.max(communitySizesDistrib));
				errorMajorCommunitySizesDistrib.append(np.std(communitySizesDistrib));
				trivialThresholdSize = trivialThreshold*verticesCount;
				if(np.max(communitySizesDistrib)>=trivialThresholdSize):
					trivialCount+=1;
				elif(len(communitySizesDistrib)>=trivialThresholdSize):
					trivialCount+=1;
			
			trivialRatios.append([trivialCount/len(communityIndices)]);
			majorCommunitySizes.append(majorCommunitySizesDistrib);
			errorMajorCommunitySizes.append(errorMajorCommunitySizesDistrib);
		distribData = np.array(majorCommunitySizes)/verticesCount;
		errorDistribData = np.array(errorMajorCommunitySizes)/verticesCount;
		if(networkName not in allDataByCategory[category]):
			allDataByCategory[category][networkName] = OrderedDict();
		if(variant not in allDataByCategory[category][networkName]):
			allDataByCategory[category][networkName][variant] = OrderedDict();
		if("MCS" not in allDataByCategory[category][networkName][variant]):
			allDataByCategory[category][networkName][variant]["MCS"] = OrderedDict();
		if("ECS" not in allDataByCategory[category][networkName][variant]):
			allDataByCategory[category][networkName][variant]["ECS"] = OrderedDict();
		if("TPR" not in allDataByCategory[category][networkName][variant]):
			allDataByCategory[category][networkName][variant]["TPR"] = OrderedDict();
		allDataByCategory[category][networkName]["LABEL"] = label;
		allDataByCategory[category][networkName][variant]["MCS"] = distribData;
		allDataByCategory[category][networkName][variant]["ECS"] = errorDistribData;
		allDataByCategory[category][networkName][variant]["TPR"] = trivialRatios;


measurements+=["MCS","ECS","TPR"];

from tqdm.auto import tqdm

dirName = "../Figures/NewCurves/%s/All"%(rewireMode);
try:
  os.makedirs(dirName)    
  print("Directory " , dirName ,  " Created ")
except FileExistsError:
  pass;

fig = plt.figure(figsize=(figureSize*1.61803398875,figureSize));
ax = fig.add_axes([0.18, 0.18, 0.65, 0.70]);
for j,variant in enumerate(variants):
  areasCategories = [];
  for category in tqdm(allDataByCategory,desc="Category"):
    areas = []
    labels = []
    networkKeys = sorted(list(allDataByCategory[category].keys()));
    for i,key in enumerate(tqdm(networkKeys,desc="Network")):
      networkData = allDataByCategory[category][key];
      networkLabel = allDataByCategory[category][key]["LABEL"];
      for j,variant in enumerate(variants):
        if(variant in networkData and measurement in networkData[variant] and networkData[variant][measurement] is not None):
          networkSize = networkSizes[key]
          values = networkData[variant][measurement]
          areas.append(1.0-np.trapz(values, alphaValues))
          labels.append(labels)
    ax.boxplot(CCAreasData,labels=labels,whis=[5,95],vert=False,showfliers=False,
    showcaps=False, showbox=False,
      boxprops={"alpha":0.25},capprops={"alpha":0.25},whiskerprops={"alpha":0.0});
  
for i in range(len(CCAreasData)):
	x = CCAreasData[i]
	y = np.random.normal(1+i, 0.04, size=len(x))
#   ax.scatter(x, y, alpha=1.0);
	plt.errorbar(x, y, xerr=CCAreasErrorsByCategory[labels[i]], fmt='o')
dirName = "../Figures/Curves/%s/%s/"%(variant,measurement);
try:
	os.makedirs(dirName)    
	print("Directory " , dirName ,  " Created ")
except FileExistsError:
	print("Directory " , dirName ,  " already exists") 

fig.savefig("%s/CcAreasBoxPlot.pdf"%(dirName));
plt.close(fig);

    fig.savefig("%s/%s_Distrib_%s.pdf"%(dirName,measurement,category));
    plt.close(fig);

      




import benchmarkParameters as bp
import createGNNetworks as gn
import utilities as ut
import importlib
import numpy as np
from multiprocessing import Process, Manager, Pool
import multiprocessing
import json
from tqdm.auto import tqdm
import pandas as pd
import xnet as xn
importlib.reload(ut)
import matplotlib.gridspec as gridspec
from adjustText import adjust_text;
import matplotlib
#Disable Matplotlib interface
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from matplotlib import patheffects
import igraph as ig

import graph_tool as gt;
import graph_tool.inference as gtInference;

importlib.reload(ut)


matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42
plt.rc('font', family='sans-serif')
# plt.rc('text', usetex=True)
plt.rc('xtick', labelsize=15)
plt.rc('ytick', labelsize=15)
plt.rc('axes', labelsize=16)
plt.rc('legend', fontsize=14)


def SBMMinimizeAndEntropy(gigraph,degreeCorrected=True):
    vertexCount = gigraph.vcount()
    directed = gigraph.is_directed()
    edges = gigraph.get_edgelist()    
    g = gt.Graph(directed=directed);
    for _ in range(0,vertexCount):
        g.add_vertex();
    for edge in edges:
        g.add_edge(edge[0],edge[1]);
    state = gtInference.minimize.minimize_blockmodel_dl(g,deg_corr=degreeCorrected);
    S1 = state.entropy();
    S2 = 0
    if("Community" in gigraph.vertex_attributes()):
        membershipProperty = g.new_vertex_property("int64_t")
        membershipProperty.a = [int(a) for a in gigraph.vs["Community"]]
        # S3 = gtInference.blockmodel.BlockState(g,B=1).entropy()
        S2 = gtInference.blockmodel.BlockState(g,b=membershipProperty,deg_corr=degreeCorrected).entropy()
    S3 = gtInference.blockmodel.BlockState(g,B=1,deg_corr=degreeCorrected).entropy()
    return (S1,S2,S3);

muValues = bp.muValues

for k in bp.ks:
    for nodesCount in bp.nodesCounts:
        benchmarkNames = [];
        S1s = []
        S2s = []
        S3s = []
        for networkRealization in range(bp.networkRealizations):
            for tau1 in bp.t1Values:
                for tau2 in bp.t2Values:
                    for mu in tqdm(muValues):
                        networkName = (bp.getNetworkName(mu,networkRealization,tau1,tau2,k=k,nodesCount=nodesCount),"$\\mu=%.3f$, $N=%d$, $k=%d$"%(mu,nodesCount,k),"Benchmarks_t1-%.2f_t2-%.2f"%(tau1,tau2))[0];
                        g = ut.getNetwork(networkName)
                        S1,S2,S3 = SBMMinimizeAndEntropy(g)
                        S1s.append(S1)
                        S2s.append(S2)
                        S3s.append(S3)


        fig = plt.figure()

        plt.plot(muValues,S1s,label="SBM")
        plt.plot(muValues,S2s,label="Planted")
        plt.plot(muValues,S3s,label="Single")

        plt.legend()
        plt.xlabel("$\mu$")
        plt.ylabel("MDL")
        plt.tight_layout()
        plt.savefig("../Figures/MDL_benchmarks_New-DC_N%d_k%d.pdf"%(nodesCount,k))
        plt.close()

        fig = plt.figure()
        plt.plot(muValues,-(np.array(S1s)-np.array(S3s))/np.array(S3s))
        # plt.legend()
        plt.xlabel("$\mu$")
        plt.ylabel("MDL Norm. Diff")
        plt.tight_layout()
        plt.savefig("../Figures/Norm_MDL_benchmarks_New-DC_N%d_k%d.pdf"%(nodesCount,k))
        plt.close()

        # print(SBMMinimizeAndEntropy(g))



gnS1s = []
gnS2s = []
gnS3s = []
for networkRealization in range(bp.networkRealizations):
    for mu in tqdm(gn.muValues):
        # benchmarkNames.append(gn.getNetworkName(gn.P_in,P_out,networkRealization));
        networkName = (gn.getNetworkName(gn.nodesCount,gn.communitiesCount,gn.avgk,mu,networkRealization),"$\\mu=%.3f$"%mu,"GN")[0]
        g = ut.getNetwork(networkName)
        g.vs["Community"] = [i%gn.communitiesCount for i in range(g.vcount())]
        S1,S2,S3 = SBMMinimizeAndEntropy(g)
        gnS1s.append(S1)
        gnS2s.append(S2)
        gnS3s.append(S3)


fig = plt.figure()

plt.plot(gn.muValues,gnS1s,label="SBM")
plt.plot(gn.muValues,gnS2s,label="Planted")
plt.plot(gn.muValues,gnS3s,label="Single")

plt.legend()
plt.xlabel("$\mu$")
plt.ylabel("MDL")
plt.tight_layout()
plt.savefig("../Figures/MDL_GN-DC.pdf")
plt.close()


fig = plt.figure()
plt.plot(muValues,-(np.array(gnS1s)-np.array(gnS3s))/np.array(gnS3s))
# plt.legend()
plt.xlabel("$\mu$")
plt.ylabel("MDL Norm. Diff")
plt.tight_layout()
plt.savefig("../Figures/Norm_MDL_GN-DC.pdf")
plt.close()


# networkName = (gn.getNetworkName(gn.nodesCount,gn.communitiesCount,gn.avgk,0.1,networkRealization),"$\\mu=%.3f$"%mu,"GN")[0]
# g = ut.getNetwork(networkName)
# g.vs["Community"] = [i%gn.communitiesCount for i in range(g.vcount())]
# xn.igraph2xnet(g,"test.xnet")




selectedNetworks = {
    # "soc-advogato",
    # "bio-dmela",
    # "OClinks_w",
    # "ia-fb-messages",
    # "fb-pages-public-figure",
    # "socfb-nips-ego",
    # "RVOR",
    # "mammalia-voles-kcs-trapping",
    # "tech-routers-rf",
    # "econ-orani678",
    # "socfb-Oberlin44",
    # "econ-poli",
    # "Airports",
    # "bio-yeast-protein-inter",
    # "power-eris1176",
    # "Facebook_combined",
    # "fb-pages-politician",
    # "web-EPA",
    # "ca-Erdos992"
    # "bio-CE-HT",
    # "mammalia-voles-rob-trapping",
    # "road-euroroad",
    # "USairport_2010",
    # "bio-yeast-protein-inter",
    # "netscience",
    # "Facebook_combined",
    # "fb-pages-politician",
    # "ca-Erdos992",
    # "tech-WHOIS",
    # "bio-CE-HT",
    # "mammalia-voles-rob-trapping",
    # "bio-dmela",
    # "bio-WormNet-v3-benchmark",
    # "road-euroroad",
    # "WesternUS_PowerGrid",
    # "Airports",
    # "power-eris1176",
    # "econ-poli",
    # "netscience",
    # "tech-WHOIS",
    # "econ-orani678",
    # "ca-Erdos992",
    # "ia-fb-messages",
    # "facebook_combined",
    # "bio-DR-CX",
    "CA-HepPh",
}

def SBMMinimizeMembership(gigraph,degreeCorrected=True):
    vertexCount = gigraph.vcount()
    directed = gigraph.is_directed()
    edges = gigraph.get_edgelist()    
    g = gt.Graph(directed=directed);
    for _ in range(0,vertexCount):
        g.add_vertex();
    for edge in edges:
        g.add_edge(edge[0],edge[1]);
    state = gtInference.minimize.minimize_blockmodel_dl(g,deg_corr=degreeCorrected);
    return list(state.get_blocks());

for networkName in selectedNetworks:
    g = xn.xnet2igraph("../Data/Networks/%s.xnet"%networkName)
    membership = SBMMinimizeMembership(g)
    g.vs["SBM Membership"] = [str(entry) for entry in membership]
    xn.igraph2xnet(g,"../Data/SelectedNetworks/%s.xnet"%networkName)
